import * as service from '../scss/admin.scss';
import CustomerBookings from './components/customer-bookings.mjs';

(function (_w, _d) {
    let urlParams = new URLSearchParams(window.location.search);
    let actions = {
        'wpcb_settings.php': eventsForm,
        'cb_form_settings.php': optionForm,
        'cb_link_generator.php': cb2,
        'cb_shortcode_generator.php': shortCodeGenerator,
        'customer_bookings.php': customerBookingsPage,
    };

    let action = actions[ urlParams.get('page') ];
    if(action){ action(); }

    _w.addEventListener('keydown', function(evt){
        // combination of alt + d and for mac cmd + d
        if(evt.key === 'd' && (evt.altKey === true || evt.metaKey === true)){
            evt.preventDefault();
            //reload window adding an extra parameter to the url
            _w.location.href = _w.location.href + '&debug_mode'
            return false;
        }
    });

    _w.addEventListener('click', function(evt){
        if(evt.target.id.indexOf('delete-wpcb-caches') > -1){
            evt.preventDefault();

            let answer = prompt('Are you sure you want to delete all caches? Type "yes" to confirm.', 'no');
            if(answer === 'yes'){
                DeleteTransients().then( (data) => {
                    _d.querySelector('.jims-d-menu-toolbar .notifications').innerHTML = `caches deleted`;
                });
            }
            return false;
        }
    });

    function customerBookingsPage(){
        const customerBookingsData = new CustomerBookings();

        if(window.graphData !== undefined) {
            customerBookingsData.barChart('booking-overview', graphData.success, graphData.failed);
        }
    }

    /**
     *
     */
    function optionForm(){
        const endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form`;
        const admin_endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form_admin`;

        _d.querySelector('#jims-form').addEventListener('submit', (evt) => {
            evt.preventDefault();
            let formdata = new FormData(evt.target);
            evt.target.querySelector('input[type=submit]').classList.add('disabled');
            SaveData( formatMessage(formdata), evt.target );

            return false;
        });

        _d.querySelector('#delete-form').addEventListener('click', (evt) => {
            evt.preventDefault();
            _d.querySelector('#delete-form').classList.add('disabled');
            DeleteForm();

            return false;
        });


        // _d.querySelector('#refresh-form').addEventListener('click', (evt) => {
        //     evt.preventDefault();
        //     _d.querySelector('#refresh-form').classList.add('disabled');
        //     RefreshForm( document.querySelector('#jims-form') );
        //
        //     return false;
        // });

        async function SaveData(data, formElement) {
            const rawResponse = await fetch(endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            const content = await rawResponse.json();
            formElement.querySelector('input[type=submit]').classList.remove('disabled');
            console.log(content);
        }

        async function DeleteForm(){
            if(typeof wpcbSecret == 'undefined') return;

            const rawResponse = await fetch(admin_endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({ 'allow' : wpcbSecret})
            });
            const content = await rawResponse.json();
            _d.querySelector('#delete-form').classList.remove('disabled');
            console.log(content);
        }

        async function RefreshForm(formElement) {

            const rawResponse = await fetch( admin_endpoint, {
                method: 'GET',
                headers: {
                    'Accept': 'application/json',
                },
            });
            const content = await rawResponse.json();
            if(content.hasOwnProperty('fields') && content.fields.length > 0){
                Object.keys(content.fields).forEach( key => {
                    if(formElement && formElement.elements[key]){
                        let field = formElement.elements[key];
                        if(input){
                            field.value = content.fields[key];
                        }
                    }
                } );
            }

            _d.querySelector('#refresh-form').classList.remove('disabled');
            console.log(content);
        }
    }

    function eventsForm(){
        const endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form-events`;
        const adminEndpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form-events-admin`;

        const form = _d.querySelector('#jims-events-form');


        form.addEventListener('submit', (evt) => {
            evt.preventDefault();
            let formdata = new FormData(evt.target);
            evt.target.querySelector('input[type=submit]').classList.add('disabled');
            SaveData( formatMessage(formdata), evt.target );

            return false;
        });


        form.querySelector('#delete-form').addEventListener('click', (evt) => {
            evt.preventDefault();
            _d.querySelector('#delete-form').classList.add('disabled');
            DeleteOptions().then( () => {
                form.reset();
            });

            return false;
        });

        // form.querySelector('#clear-form').addEventListener('click', (evt) => {
        //     form.reset();
        // });

        async function DeleteOptions(){
            if(typeof wpcbSecret == 'undefined') return;

            const rawResponse = await fetch(adminEndpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({ 'allow' : wpcbSecret})
            });
            const content = await rawResponse.json();
            _d.querySelector('#delete-form').classList.remove('disabled');

            return rawResponse;
        }

        async function SaveData(data, formElement) {
            const rawResponse = await fetch(endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            const content = await rawResponse.json();
            formElement.querySelector('input[type=submit]').classList.remove('disabled');
            console.log(content);
        }
    }

    /**
     *
     */
    function cb2(){
        const d = document;
        let data = service();

        let generator = d.querySelector("#generator-2");
        generator.addEventListener("submit", CalculateLink);


        function CalculateLink(evt) {
            evt.preventDefault();
            let formdata = new FormData(generator);
            let params   = new URLSearchParams(formdata);
            let parent = jims_wpcb_variables.home_url;
            let logoUrl = '';

            let panel    = d.querySelector('.cb-link');
            for (var pair of params.entries()) {

                if(pair[0].indexOf('logo') > -1){
                    logoUrl = pair[1];
                    params.delete(pair[0]);
                }

                if(pair[0].indexOf('Color') > -1 ){
                    params.set(pair[0], pair[1].replace('#', ''))
                }

                if (!pair[1] || pair[1].length === 0) {
                    params.delete(pair[0])
                }
            }

            let lnk = `${jims_wpcb_variables.http_schema}://${jims_wpcb_variables.domain}/?${params.toString()}`;
            lnk += `&logoUrl=${logoUrl}`;

            if(parent.length > 3){
                lnk += `&parent=${parent}`;
            }

            _d.querySelector('.notification').classList.remove('hide');
            panel.querySelector('pre').innerText = lnk;
            panel.classList.remove('hidden')
            return !1
        }


        data.divisions.then( listobj => {
            populatedOptionField(
                d.getElementById('division-id'),
                listobj,
                'id', 'name',
                'Select your division'
            );
        } );

        d.getElementById('division-id').addEventListener('change', function(evt){
            if(evt.target.value >= 1) {
                data.services.then( listobj => {

                    let list = listobj.filter( obj => {
                        return obj.group_id == evt.target.value
                    });

                    populatedOptionField(
                        d.getElementById('service-id'),
                        list,
                        'id', 'friendly_name',
                        'Select the service'
                    );
                });
            }
        });
    }

    function shortCodeGenerator(){
        const d = document;
        let data = service();
        let panel    = d.querySelector('.cb-link');
        let button =  _d.querySelector('.jims-main form input[type=submit]');

        _w.addEventListener('load', function(evt){
            data.services.then( listobj => {

                let list = listobj.filter( obj => {
                    return obj.group_id === parseInt(jims_wpcb_variables.embeddedFormOptions.DivisionId);
                });

                populatedOptionField(
                    d.getElementById('service-id'),
                    list,
                    'id', 'friendly_name',
                    'Select the service'
                );
            });

            button.classList.remove('disabled');
            document.querySelector('.jims-main form').addEventListener('submit', (evt)=>{
                evt.preventDefault();

                generateShortcode();
                return false;
            });

        });

        _d.querySelector('form[name=native-booking-form-shortcode-generator]').addEventListener('change', (evt)=>{
            let form = _d.querySelector('form[name=native-booking-form-shortcode-generator]');
            console.dir(form);
            let formdata = new FormData(form);
            let serviceIds = [];

            for (var pair of formdata.entries()) {
                //delete if param empty
                if (!pair[1] || pair[1].length === 0) {
                    formdata.delete(pair[0]);
                }

                //get the service ids
                if(pair[0].indexOf('available_service') > -1){
                    serviceIds.push(pair[1]);
                }
            }

            //create a string of the formdata like key=value key=value
            let params   = new URLSearchParams(formdata);

            params.forEach((value, key) => {
                 if(key.indexOf('available_service') > -1){
                    params.delete(key);
                }
            });

            params.set('service_ids', `"${serviceIds.join(',')}"`);
            console.log(serviceIds);

            //show the shortcode
            _d.querySelector('#native-booking-form-shortcode').innerText = `[${_w.nativeBookingFormShortCode} ${ replaceAsciiWithChar(params.toString().replace(/&/g, ' '))}]`;
        });

        function replaceAsciiWithChar(str){
            let string = str.replace(/&#(\d+);/g, function(match, dec) {
                return String.fromCharCode(dec);
            });

            //remove url encoded spaces and commas
            return string.replace(/%20/g, ' ').replace(/%2C/g, ',').replace(/%22/g, '"');
        }

        function generateShortcode(){
            let properties = {
                'division_id': jims_wpcb_variables.DivisionId,
                'service_id': d.getElementById('service-id').value
            };
            let attributes = [];
            for(let index in properties){ if(properties[index].length) attributes.push( `${index}='${properties[index]}'` ); }

            _d.querySelector('.notification').classList.remove('hide');
            panel.classList.remove('hidden')
            panel.querySelector('pre').innerText = `[jims_wpcb_form ${attributes.join(' ')}]`;
        }
    }


    function populatedOptionField(target, listObject, ValueName, LabelName, placeholder){
        let template = '<option value="{{val}}">{{label}}</option>';
        let html = [];
        html.push(`<option value="" selected>${placeholder}</option>`);

        listObject.forEach( element => {
            html.push( template.replace('{{val}}', element[ValueName] ).replace('{{label}}', element[LabelName] ) );
        });

        target.innerHTML = html.join('');
    }

    function formatMessage(formData) {
        let message = {};
        for (var pair of formData.entries()) {
            message[pair[0]] = pair[1];
        }

        return message;
    }

    /**
     * Service Class
     *
     * @returns {{services: Promise<any>, divisions: Promise<any>}}
     */
    function service(){
        let config = {
            divisionsLink : `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/service/division`,
            servicesLink : `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/service/division/services`
        }

        async function call(link){
            const response = await fetch(link);
            return response.json();
        }

        return {
            'divisions': call(config.divisionsLink),
            'services': call(config.servicesLink)
        }
    }

    async function DeleteTransients(){
        /**
         * method post, path: /jims-wpcb/v1/options/clear-transients
         */
        const rawResponse = await fetch(`${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/clear-transients`, {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        });
        const content = await rawResponse.json();
        console.log(content);

        return rawResponse;
    }
}(window, document));