<?php

namespace JimsCustomerBooking\Services;

use JimsCustomerBooking\Helpers\Http;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Validators\JsonValidator;

if (!class_exists ('\JimsCustomerBooking\Services\DivisionService')) {
    /**
     * TODO: Update this to use FMS API instead of internal.jims.net
     */
    class DivisionService extends Service
    {

        private $serviceLink = 'https://api.jims.net/api/CustomerBookings/get-initial-booking-request';

        private $divisionsLink = JIMS_INTERNAL_URL . '/api/division/all';

        private $countriesLink = JIMS_INTERNAL_URL . '/api/division/countries/active/all';

        protected $requestData = [
            "confirmed_not_previous_client" => true,
            "userInfo" => [],
            "bookingStep" => [
                "title" => "Select Service",
                "pageIndex" => 1,
                "progress" => 30,
                "url" => "servicesChoose"
            ],
            "brandSource" => "Jims",
            "jimsProvider" => "Jims",
            "jimsProperty" => "CB",
            "utmSource" => "",
            "utmMedium" => "",
            "utmCampaign" => "",
            "utmTerm" => "",
            "utmContent" => "",
            "otherServices" => "CB_Book",
            "countryCode" => "AU"
        ];

        public function __construct($serviceLink = false)
        {
            if (is_string ($serviceLink) && (strlen ($serviceLink) > 3)) {
                $this->serviceLink = $serviceLink;
            }

            return $this;
        }

        /**
         * @return void
         */
        public function call()
        {
            switch ($_GET['part']) {
                case 'divisions':
                    return $this->getDivisions ();
                case 'services':
                    return $this->getServices ();
            }
        }

        /**
         * @param $link
         * @return mixed
         */
        public function getDivisions($short = true)
        {
            $output = [];
            if (false === ($value = get_transient (StringHelper::GetPrefixed ('divisions')))) {
                $curl = curl_init ();

                curl_setopt_array ($curl, [
                    CURLOPT_URL => $this->divisionsLink,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => 'GET',
                ]);
                $response = curl_exec ($curl);
                curl_close ($curl);
                set_transient (StringHelper::GetPrefixed ('divisions'), $response, 5 * DAY_IN_SECONDS);
                $output = json_decode ($response);
            } else {
                $output = json_decode ($value);
            }


            $short_output = [];
            if ($short && !empty($output)) {
                foreach ($output as $division) {
                    array_push ($short_output, [
                        'id' => $division->id,
                        'uid' => $division->uid,
                        'name' => $division->name
                    ]);
                }

                return $short_output;
            }

            return $output;
        }

        /**
         * @return mixed
         */
        public function getServices($format = false)
        {
            if (false === ($output = get_transient (StringHelper::GetPrefixed ('services')))) {

                $output = $this->httpget ($this->serviceLink);

                if(JsonValidator::isNotEmptyObject($output, 2)) set_transient (StringHelper::GetPrefixed ('services'), $output, 1 * DAY_IN_SECONDS);

                return $output;
            }

            return $format === true ? $this->formatServices ($output) : $output;
        }

        public function getCountries()
        {
            if (false === ($output = get_transient (StringHelper::GetPrefixed ('countries')))) {
                $output = Http::Get ($this->countriesLink)['result'];
                set_transient (StringHelper::GetPrefixed ('countries'), $output, 1 * DAY_IN_SECONDS);
            }

            return json_decode ($output);
        }

        /**
         * @param $services
         * @return array|array[]
         */
        public function formatServices($services)
        {
            $services_arr = $services->payload->jimsServices;

            return array_map (function ($service) {
                return [
                    'name' => $service->friendly_name,
                    'id' => $service->id,
                    'division_id' => $service->group_id
                ];
            }, $services_arr);
        }

        /**
         *
         */
        protected function httpget($link = false)
        {
            $target = $this->serviceLink;
            if($link){
                $target = $link;
            }

            $curl = curl_init ();
            curl_setopt_array ($curl, array(
                CURLOPT_URL => $target,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => json_encode ($this->requestData),
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json',
                    'Authorization: Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsIng1dCI6Ik50czBJSWRUYUd5YU5EYWxCWnRtR2ZXX2R0TSIsImtpZCI6Ik50czBJSWRUYUd5YU5EYWxCWnRtR2ZXX2R0TSJ9.eyJpc3MiOiJodHRwczovL2lkLWludC5jbG91ZC5qaW1zLm5ldC9pZGVudGl0eSIsImF1ZCI6Imh0dHBzOi8vaWQtaW50LmNsb3VkLmppbXMubmV0L2lkZW50aXR5L3Jlc291cmNlcyIsImV4cCI6MTYzNTM4MTcxNCwibmJmIjoxNjM1Mzc4MTE0LCJjbGllbnRfaWQiOiJmbXM0X3NlcnZpY2UiLCJzY29wZSI6ImppbXNBcGkifQ.DXl5O5-4LfLbenhdiMoifLfygHPQYhtP-T7L4a_5wBLsx71lkm2GRSExkWQUxjSM1zi1_H7kgwwWuOj2zRW5VX9DI8PTvy2YrHH7eTCpfu_vXil8W7xrqtHahCY7lGpK5b_nvavmIwsTCpK4PgKqI87u_UazCW5SpHcGBKbDtbpn3TY7O6UehBw4Ty3wPFD97iqAjqHggmXrrTlJtz79u4kpYcC6DH-AbXARA90HIShlZf8PWtuoJFc2HgCYT3eELkCmPm-5u4wLPeOAdqpuUNBT4HBAivhrekIaK9RW3nbpYjoVnjFm_nqIjFrkN4pI4LJ2SDI3omrcKs7GCOJFLw'
                ),
            ));

            $response = curl_exec ($curl);
            curl_close ($curl);

            return StringHelper::isJson($response) ? json_decode ($response) : '';
        }
    }
}
