<?php

namespace JimsCustomerBooking\Shortcodes;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Template;

class BookingFormShortCode extends Shortcode
{

    /**
     * @var string
     */
    protected $template = 'shortcode.booking_form';

    protected $default_attributes = [];

    /**
     * @inheritDoc
     */
    public function Configuration()
    {
        return [
            'shortcode_id' => StringHelper::GetPrefixed('booking_form', 'machine'),
            'label' => 'Booking Form',
            'template' => $this->template
        ];
    }

    /**
     * @inheritDoc
     */
    public function Handler($atts, $content)
    {
        $component_id = mt_rand(111, 9999);
        $css_classes = [ 'wpcb', 'wpcb-form__native', $this->shortcode_id . '-' . $component_id ];

        $_atts = $this->GetAtts([
            'division_id' => $this->getFormOption('bookingFormDivisionId'),
            'phone_number' => $this->getFormOption('phone'),
            'service_ids' => [],
            'component_id' => $component_id,
            'logo_url' =>  apply_filters( StringHelper::getPrefixed('booking_form_logo_url'), $this->getLogoUrl() ),
            'hide_logo' => 0,
            'contained' => 0,
            'images_base_url' => JIMS_CBP_URL . 'assets/dist/img',
            'class' => implode(' ', $css_classes),
            'prefix_class' => StringHelper::getPrefix (StringHelper::PREFIXED_TYPE_SLUG),
            'button_text' => 'Book Now'
        ]);

        if(isset($atts['class']) && !empty($atts['class'])){
            $atts['class'] =  implode(' ', $css_classes) . ' ' . $atts['class'];
        }
        if (isset($atts['service_ids']) && !empty($atts['service_ids'])) {
            $atts['service_ids'] = explode(',', $atts['service_ids']);
        }

        // apply filters, modify attributes
        $atts = apply_filters (
            StringHelper::getPrefixed ('booking_form_atts'),
            shortcode_atts($_atts, $atts, $this->shortcode_id ),
            $content, $this->shortcode_id);


        if (isset($atts['contained']) && $atts['contained'] == 1) {
            $atts['class'] .= ' contained';
        }



        // get template object, it contains override if the shortcode has template attribute
        $template = new Template($atts['template'] ?? $this->template);

        // render template, allows overriding the html
        return apply_filters (
            StringHelper::GetPrefixed ('form_html'),
            $template->get( $atts + [ 'content' => $content] ),
            $atts, $content, $this->shortcode_id
        );
    }

    public function getFormOption($name, $default = ''){
        $options = OptionHelper::get(NativeFormSettingsPage::$option_name, [] );
        return $options[$name] ?? $default;
    }

    /**
     * @return string
     */
    public function getLogoUrl(): string {
        $logo = '';

        if( \has_custom_logo() ) {
            $custom_logo_id = get_theme_mod( 'custom_logo' );
            $image = wp_get_attachment_image_src( $custom_logo_id , 'full' );
            $logo = is_array ( $image ) && count ( $image ) > 0 ? $image[0] : '';
        }

        if( empty($logo) ){
            $logo = JIMS_CBP_URL . 'assets/dist/img/jims-logo.png';
        }

        return $logo;
    }
}