<?php
/**
 * @var string $title
 * @var array $fields
 * @var array $services
 * @var string $css_class
 * @var \JimsCustomerBooking\Jims_Options $jims_options
 * @var $selected_division int
 * @var $pages     array
 * @var $parts     array
 */

use JimsCustomerBooking\Helpers\StringHelper;

/**
 * @var $header \JimsCustomerBooking\Template
 * @var $footer \JimsCustomerBooking\Template
 */
$header = $footer = null;
extract($parts);

$native_services = $services;
?>


<?php if (!empty($header)): ?>
    <?php $header->render([
        'title' => 'Shortcode Generator',
        'description' => 'This page allows to generate shortcodes for all CB versions',
        'pages' => $pages,
        'url' => get_template_directory_uri()
    ], true); ?>
<?php endif; ?>

<style>
    .service-container {
        font-size: 16px;
    }
    .service-container .available-services {
        background-color:white;
        padding: 1em;
    }

    .service-container > label {
        font-size: 1rem;
    }

    .service-container .service input {
        display: none;
    }

    .service-container .service label {
        display: block;
        padding: 7px 3px;
        margin: 0;
        position: relative;
    }

    .service-container .service label:hover,
    .service-container .service label:focus {
        background-color: #00000011;
        cursor: pointer
    }

    .service-container .service label:before {
        position: absolute;
        content: "";
        width: 1em;
        height: 1em;
        background-color: transparent;
        border: 1px solid #00000055;
        right: 3px;
        top: 5.33px;
        border-radius: 0.5em;
    }


    .service-container .service input:checked + label:after {
        position: absolute;
        content: "";
        width: 0.8em;
        height: 0.8em;
        background-color: #00da7d;
        right: 5.6px;
        top: 7.9px;
        border-radius: 0.5em;
    }
</style>

<div class="<?php echo $css_class ?>">
    <section class="jims-main">
        <script>
            window.nativeBookingFormShortCode = '<?php echo StringHelper::GetPrefixed('booking_form', 'machine'); ?>';
        </script>

        <div class="sub-section">
            <h3>Generator CB3</h3>
            <p>To add the CB3 form shortcode just copy / paste the following:</p>

            <form name="native-booking-form-shortcode-generator">
                <label for="hide-logo">Hide Logo
                    <input type="checkbox" name="hide_logo" id="hide-logo" value="1">
                </label>
                <label for="contained">Contained (disable if inside popup)
                    <input type="checkbox" name="contained" id="contained" value="1">
                </label>

                <div class="service-container">
                    <?php if (is_array($services) and count($services) > 0): ?>
                        <label for="available-service">Available Services</label>
                        <div class="available-services">
                            <?php foreach ($native_services as $native_service): ?>
                                <?php if (empty($native_service['name']) || $native_service['division_id'] != $native_fields['bookingFormDivisionId']) continue; ?>
                                <div class="service">
                                    <input type="checkbox" class="available-service" name="available_service[]"
                                           value="<?php echo $native_service['id'] ?>"
                                           id="available-service-<?php echo $native_service['id'] ?>">
                                    <label for="available-service-<?php echo $native_service['id'] ?>"><?php echo $native_service['name'] ?> <small style="display: inline; font-weight: lighter;">ID: <?php echo $native_service['id'] ?></small></label>
                                </div>
                            <?php endforeach; ?>
                        </div>
                    <?php else: ?>
                        <div class="alert alert-warning" role="alert">
                            <strong>Warning!</strong> No services found. Please create a service first.
                        </div>
                    <?php endif; ?>
                </div>
            </form>

            <pre style="white-space: normal; word-break: break-all; outline: darkblue; background: darkcyan; color: white; border-radius: 10px; padding: 1em 0.5em;"
                 id="native-booking-form-shortcode">
                <?php echo '[' . StringHelper::GetPrefixed('booking_form', 'machine') . ']' ?>
            </pre>
        </div>

        <div class="sub-section">
            <h3>Generator for CB1 and CB2</h3>
            <p>
                This tool provides an easy way to generate a CB1 or CB2 shortcode, depending on your settings.
            </p>

            <form>
                <div class="mb-3">
                    <label for="service-id">Service</label>
                    <select type="text" name="ServiceId" class="widefat" id="service-id" placeholder="Select a service">
                        <option value="">Select Service</option>
                    </select>
                    <small class="form-text text-muted">
                        Select you service by name, optional
                    </small>
                </div>

                <input type="submit" class="button button-primary disabled" value="Create Shortcode">

                <div class="notification hide" style="margin-top: 2em;">
                    <div class="alert alert-dark cb-link hidden" role="alert">
                        <pre style="white-space: normal; word-break: break-all;"></pre>
                    </div>
                </div>
            </form>
        </div>
    </section>
</div>
</div>
</div>
</div>