<?php

namespace JimsGroup\Datatypes;

/**
 * Class BookingRequestType
 *
 * @package JimsGroup\Datatypes
 * List of all the properties that can be set in the BookingRequestType
 * @property int $pageIndex
 * @property int $progress
 * @property string $title
 * @property string $url
 * @property string $brandSource
 * @property bool $confirmed_not_previous_client
 * @property string $countryCode
 * @property string $jimsProperty
 * @property string $jimsProvider
 * @property string $otherServices
 * @property array $userInfo
 * @property string $utmCampaign
 * @property string $utmContent
 * @property string $utmMedium
 * @property string $utmSource
 * @property string $utmTerm
 */
abstract class BookingRequestType extends Datatype
{
    /**
     * @var array
     */
    protected $default = [
        'id' => 0,
        "addressId" => -1,
        'bookingStep' => null,
        'pageIndex' => 1,
        'progress' => 30,
        'title' => 'Select Service',
        'url' => 'servicesChoose',
        'brandSource' => 'Jims',
        'confirmed_not_previous_client' => true,
        'countryCode' => 'AU',
        'jimsProperty' => 'CB30',
        'jimsProvider' => 'Jims',
        'otherServices' => 'CB30_Book',
        'division_id' => 0,
        'bookingAddress' => null,
        'serviceLine' => null,
        'apiKey' => '',
        'selectedServicesIds' => [],
        'ipAddress' => '127.0.0.1',
        'userInfo' => array(),
        'utmCampaign' => '',
        'utmContent' => '',
        'utmMedium' => '',
        'utmSource' => '',
        'utmTerm' => '',
    ];

    /**
     * @var array
     */
    protected $data = [];

    /**
     * @param $name
     * @return mixed
     */
    public function __get( $name )
    {
        return $this->data[$name] ?? null;
    }

    /**
     * @param $name
     * @param $value
     * @return void
     */
    protected function set( $name, $value )
    {
        $this->data[$name] = $value;
    }

    /**
     * @param array $data
     */
    protected function update( $data )
    {
        $this->data = array_merge( $this->data, $data );
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return json_encode( $this->data );
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $data = $this->data;
        foreach ( $data as $key => $value ) {
            if ( $value instanceof Datatype ) {
                $data[$key] = $value->toArray();
            }
        }

        return $data;
    }

    public function validate(array $data)
    {
        return parent::validate ($data); // TODO: Change the autogenerated stub
    }

    public function setCountryCode($countryCode)
    {
        $this->set ('countryCode', $countryCode);
    }
}