<?php

namespace JimsGroup\Utilities;

/**
 *
 */
class HttpRequest
{

    protected $query_encoding = \PHP_QUERY_RFC1738;

    /**
     * @var string
     */
    public function __construct()
    {
        $this->url = '';
        $this->method = 'GET';
        $this->headers = [];
        $this->body = '';
        $this->response = '';
        $this->responseCode = 0;
        $this->responseHeaders = [];
    }

    /**
     * @param string $url
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }

    /**
     * @param string $method
     */
    public function setMethod($method)
    {
        $this->method = $method;
    }

    /**
     * @param array $headers
     */
    public function setHeaders($headers)
    {
        $this->headers =  $headers;
    }

    public function addHeader($header)
    {
        $this->headers[] = $header;
    }

    /**
     * @param string $body
     */
    public function setBody($body)
    {
        $this->body = $body;
    }

    /**
     * @return string
     */
    public function getResponse()
    {
        return $this->response;
    }

    /**
     * @return int
     */
    public function getResponseCode()
    {
        return $this->responseCode;
    }

    /**
     * @return array|string
     */
    public function getResponseHeaders($toArray = false)
    {
        return !!$toArray ? $this->headersToArray ($this->responseHeaders) : $this->responseHeaders;
    }

    /**
     * @return string
     */
    public function send()
    {
        $ch = \curl_init();

        //define the request
        curl_setopt_array ($ch, [
            CURLOPT_URL => $this->url,
            CURLOPT_CUSTOMREQUEST => $this->method,
            CURLOPT_HTTPHEADER => $this->headers,
            CURLOPT_POSTFIELDS => $this->body,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HEADER => true,
        ]);

        error_log($this->body, 0);

        $response = \curl_exec($ch);

        error_log($this->response, 0);

        //parse the response
        $header_size = \curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $this->responseHeaders = \substr($response, 0, $header_size);
        $this->response = \substr($response, $header_size);
        $this->responseCode = \curl_getinfo($ch, CURLINFO_HTTP_CODE);

        \curl_close($ch);
    }

    /**
     * @param string $url
     * @param array $params
     * @return string
     */
    public function get($url)
    {
        $this->setUrl($url);
        $this->setMethod('GET');
        $this->send();
        return $this->getResponse();
    }

    /**
     * @param string $url, the url to post to
     * @param array $params, the data to be encoded
     * @param string $enctype, how the data should be encoded
     * @return string
     */
    public function post($url, $params, $enctype = 'application/x-www-form-urlencoded'){

        $body = $this->formatBody ($enctype, $params);

        $this->setUrl($url);
        $this->setMethod('POST');

        //set headers
        $this->addHeader('Content-Type: ' . $enctype);
        $this->addHeader('Content-Length: ' . strlen($body));

        $this->setBody($body);
        $this->send();

        return $this->getResponse();
    }

    /**
     * @param mixed $enctype, how the data should be encoded
     * @param array $params, the data to be encoded
     * @return string
     */
    protected function formatBody(mixed $enctype, array $params)
    {
        $body = '';
        if ($enctype == 'application/x-www-form-urlencoded') {
            $body = \http_build_query ($params, '', '&', \PHP_QUERY_RFC1738);
        } else if ($enctype == 'multipart/form-data') {
            $body = $params;
        } else if ($enctype == 'application/json') {
            $body = \json_encode ($params);
        }

        return $body;
    }

    /**
     * @param $jwt_token
     * @return array
     */
    public static function parseJWT($jwt_token){
        $jwt_token = explode('.', $jwt_token);
        $jwt_header = json_decode(base64_decode($jwt_token[0]), true);
        $jwt_payload = json_decode(base64_decode($jwt_token[1]), true);
        $jwt_signature = $jwt_token[2];

        return array(
            'header' => $jwt_header,
            'payload' => $jwt_payload,
            'signature' => $jwt_signature
        );
    }

    /**
     * @param $payload
     * @param $secret
     * @return string
     */
    public static function createJWT($payload, $secret){
        $header = array(
            'alg' => 'HS256',
            'typ' => 'JWT'
        );
        $header = base64_encode(json_encode($header));
        $payload = base64_encode(json_encode($payload));
        $signature = hash_hmac('sha256', $header . '.' . $payload, $secret, true);
        $signature = base64_encode($signature);
        $jwt = $header . '.' . $payload . '.' . $signature;
        return $jwt;
    }

    /**
     * @param int $query_encoding
     */
    public function setQueryEncoding(int $query_encoding): void
    {
        $this->query_encoding = $query_encoding;
    }

    public function headersToArray($data){
        $headers = [];
        if(is_array($data)){ return $data; }

        //extract the http status
        $status_arr = $this->getStatusFromHeadersString ($data);

        $headers_arr = explode("\n", $data);
        foreach ($headers_arr as $header){
            $header = explode(':', $header);
            if(count($header) == 2){
                $headers[$header[0]] = trim($header[1]);
            }
        }
        return $status_arr + $headers;
    }

    /**
     * @param $data
     * @param $matches
     * @return mixed
     */
    protected function getStatusFromHeadersString($data): mixed
    {
        preg_match ('/HTTP\/.* ([0-9]{3})/', $data, $matches);
        return count($matches) > 0 ? [
            'status' => $matches[1] ?? $matches[0],
        ]: [];
    }
}